@@ -21,27 +21,32 @@ from hexbytes import (
 from web3._utils.compat import (
     Literal,
 )
-from web3._utils.threads import (
-    Timeout,
+from web3._utils.utility_methods import (
+    all_in_dict,
+    any_in_dict,
 )
-from web3.exceptions import (
-    TransactionNotFound,
+from web3.constants import (
+    DYNAMIC_FEE_TXN_PARAMS,
 )
 from web3.types import (
     BlockIdentifier,
     TxData,
     TxParams,
-    TxReceipt,
     Wei,
     _Hash32,
 )
 
-TX_PARAM_LITERALS = Literal['from', 'to', 'gas', 'gasPrice', 'value', 'data', 'nonce', 'chainId']
+TX_PARAM_LITERALS = Literal['type', 'from', 'to', 'gas', 'maxFeePerGas', 'maxPriorityFeePerGas',
+                            'gasPrice', 'value', 'data', 'nonce', 'chainId', 'accessList']
 
 VALID_TRANSACTION_PARAMS: List[TX_PARAM_LITERALS] = [
+    'type',
     'from',
     'to',
     'gas',
+    'accessList',
+    'maxFeePerGas',
+    'maxPriorityFeePerGas',
     'gasPrice',
     'value',
     'data',
@@ -54,6 +59,11 @@ TRANSACTION_DEFAULTS = {
     'data': b'',
     'gas': lambda web3, tx: web3.eth.estimate_gas(tx),
     'gasPrice': lambda web3, tx: web3.eth.generate_gas_price(tx) or web3.eth.gas_price,
+    'maxFeePerGas': (
+        lambda web3, tx:
+        web3.eth.max_priority_fee + (2 * web3.eth.get_block('latest')['baseFeePerGas'])
+    ),
+    'maxPriorityFeePerGas': lambda web3, tx: web3.eth.max_priority_fee,
     'chainId': lambda web3, tx: web3.eth.chain_id,
 }
 
@@ -79,39 +89,36 @@ def fill_transaction_defaults(web3: "Web3", transaction: TxParams) -> TxParams:
     """
     if web3 is None, fill as much as possible while offline
     """
+    strategy_based_gas_price = web3.eth.generate_gas_price(transaction)
+    is_dynamic_fee_transaction = (
+        not strategy_based_gas_price
+        and (
+            'gasPrice' not in transaction  # default to dynamic fee transaction
+            or any_in_dict(DYNAMIC_FEE_TXN_PARAMS, transaction)
+        )
+    )
+
     defaults = {}
     for key, default_getter in TRANSACTION_DEFAULTS.items():
         if key not in transaction:
+            if (
+                is_dynamic_fee_transaction and key == 'gasPrice'
+                or not is_dynamic_fee_transaction and key in DYNAMIC_FEE_TXN_PARAMS
+            ):
+                # do not set default max fees if legacy txn or gas price if dynamic fee txn
+                continue
+
             if callable(default_getter):
-                if web3 is not None:
-                    default_val = default_getter(web3, transaction)
-                else:
-                    raise ValueError("You must specify %s in the transaction" % key)
+                if web3 is None:
+                    raise ValueError("You must specify a '%s' value in the transaction" % key)
+                default_val = default_getter(web3, transaction)
             else:
                 default_val = default_getter
+
             defaults[key] = default_val
     return merge(defaults, transaction)
 
 
-def wait_for_transaction_receipt(
-    web3: "Web3", txn_hash: _Hash32, timeout: float, poll_latency: float
-) -> TxReceipt:
-    with Timeout(timeout) as _timeout:
-        while True:
-            try:
-                txn_receipt = web3.eth.get_transaction_receipt(txn_hash)
-            except TransactionNotFound:
-                txn_receipt = None
-            # FIXME: The check for a null `blockHash` is due to parity's
-            # non-standard implementation of the JSON-RPC API and should
-            # be removed once the formal spec for the JSON-RPC endpoints
-            # has been finalized.
-            if txn_receipt is not None and txn_receipt['blockHash'] is not None:
-                break
-            _timeout.sleep(poll_latency)
-    return txn_receipt
-
-
 def get_block_gas_limit(web3: "Web3", block_identifier: Optional[BlockIdentifier] = None) -> Wei:
     if block_identifier is None:
         block_identifier = web3.eth.block_number
@@ -141,8 +148,7 @@ def get_buffered_gas_estimate(
 def get_required_transaction(web3: "Web3", transaction_hash: _Hash32) -> TxData:
     current_transaction = web3.eth.get_transaction(transaction_hash)
     if not current_transaction:
-        raise ValueError('Supplied transaction with hash {} does not exist'
-                         .format(transaction_hash))
+        raise ValueError(f'Supplied transaction with hash {transaction_hash!r} does not exist')
     return current_transaction
 
 
@@ -152,6 +158,14 @@ def extract_valid_transaction_params(transaction_params: TxData) -> TxParams:
         for key in VALID_TRANSACTION_PARAMS
         if key in transaction_params
     })
+    # There is always a gasPrice now on eth_getTransaction call for pending transactions, including
+    # dynamic fee transactions. For dynamic fee transactions, we need to pull the gasPrice value
+    # back out of the extracted params if it is equal to the expected value (maxFeePerGas). If
+    # we don't, the modified transaction will include a gasPrice as well as dynamic fee values in
+    # the eth_sendTransaction call and cause a conflict.
+    if all_in_dict(DYNAMIC_FEE_TXN_PARAMS, extracted_params):
+        if extracted_params['gasPrice'] == extracted_params['maxFeePerGas']:
+            extracted_params.pop('gasPrice')
 
     if extracted_params.get('data') is not None:
         if transaction_params.get('input') is not None:
@@ -181,31 +195,42 @@ def assert_valid_transaction_params(transaction_params: TxParams) -> None:
 
 def prepare_replacement_transaction(
     web3: "Web3",
-    current_transaction: TxData,
-    new_transaction: TxParams,
+    original_transaction: TxData,
+    replacement_transaction: TxParams,
     gas_multiplier: float = 1.125
 ) -> TxParams:
-    if current_transaction['blockHash'] is not None:
-        raise ValueError('Supplied transaction with hash {} has already been mined'
-                         .format(current_transaction['hash']))
-    if 'nonce' in new_transaction and new_transaction['nonce'] != current_transaction['nonce']:
+    if original_transaction['blockHash'] is not None:
+        raise ValueError(f'Supplied transaction with hash {original_transaction["hash"]!r} '
+                         'has already been mined')
+    if 'nonce' in replacement_transaction and (
+        replacement_transaction['nonce'] != original_transaction['nonce']
+    ):
         raise ValueError('Supplied nonce in new_transaction must match the pending transaction')
 
-    if 'nonce' not in new_transaction:
-        new_transaction = assoc(new_transaction, 'nonce', current_transaction['nonce'])
+    if 'nonce' not in replacement_transaction:
+        replacement_transaction = assoc(
+            replacement_transaction, 'nonce', original_transaction['nonce']
+        )
 
-    if 'gasPrice' in new_transaction:
-        if new_transaction['gasPrice'] <= current_transaction['gasPrice']:
+    if any_in_dict(DYNAMIC_FEE_TXN_PARAMS, replacement_transaction):
+        # for now, the client decides if a dynamic fee txn can replace the existing txn or not
+        pass
+
+    elif 'gasPrice' in replacement_transaction and original_transaction['gasPrice'] is not None:
+        if replacement_transaction['gasPrice'] <= original_transaction['gasPrice']:
             raise ValueError('Supplied gas price must exceed existing transaction gas price')
-    else:
-        generated_gas_price = web3.eth.generate_gas_price(new_transaction)
-        minimum_gas_price = int(math.ceil(current_transaction['gasPrice'] * gas_multiplier))
-        if generated_gas_price and generated_gas_price > minimum_gas_price:
-            new_transaction = assoc(new_transaction, 'gasPrice', generated_gas_price)
-        else:
-            new_transaction = assoc(new_transaction, 'gasPrice', minimum_gas_price)
 
-    return new_transaction
+    else:
+        generated_gas_price = web3.eth.generate_gas_price(replacement_transaction)
+        minimum_gas_price = int(math.ceil(original_transaction['gasPrice'] * gas_multiplier))
+        if generated_gas_price and generated_gas_price > minimum_gas_price:
+            replacement_transaction = assoc(
+                replacement_transaction, 'gasPrice', generated_gas_price
+            )
+        else:
+            replacement_transaction = assoc(replacement_transaction, 'gasPrice', minimum_gas_price)
+
+    return replacement_transaction
 
 
 def replace_transaction(
